
classdef MokuBodeAnalyzer < moku
    % MOKUBODEANALYZER(IpAddr) 
    % Driver class for the Moku:BodeAnalyzer
    %
    % MOKUBODEANALYZER Properties:
    %   IP - IP address of the currently connected Moku:Lab device.
    %   Instrument - Name of the currently running instrument.
    %   Timeout - Command timeout period (sec)
    %
    % MOKUBODEANALYZER Methods:
    %   commit  - Apply all modified settings.
    %   gen_off  - Turn off the output sweep.
    %   get_data  - Get current sweep data.
    %   get_frontend  - Get the analog frontend configuration.
    %   get_realtime_data  - Get downsampled data from the instrument with low latency.
    %   set_defaults  - Reset the Bode Analyzer to sane defaults 
    %   set_framerate  - Set framerate 
    %   set_frontend  - Configures gain, coupling and termination for each channel.
    %   set_output  - Set the output sweep amplitude.
    %   set_sweep  - Set the output sweep parameters
    %   set_xmode  - Set rendering mode for the horizontal axis.
    %   start_sweep  - Start sweeping
    %   stop_sweep  - Stop sweeping.
    %
    % See also moku

    methods
        function obj = MokuBodeAnalyzer(IpAddr)
            obj@moku(IpAddr, 'bodeanalyzer');
        end

        function commit(obj)
            % COMMIT()
            % Apply all modified settings.
            % 
            % .. note::
            % 
            %     If the `autocommit` feature has been turned off, this function can be used to manually apply any instrument
            %     settings to the Moku device. These instrument settings are those configured by calling all *set_* and *gen_* type
            %     functions. Manually calling this function allows you to atomically apply many instrument settings at once.
            params = struct();
            mokuctl(obj, 'commit', params);
        end

        function gen_off(obj, varargin)
            % GEN_OFF()
            % GEN_OFF(...,param1,val1,param2,val2,...)
            % Turn off the output sweep.
            % 
            % If *ch* is specified, turn off only a single channel, otherwise turn off both.
            % 
            % :type ch: int; {1,2}
            % :param ch: Channel number to turn off (None, or leave blank, for both)
            params = struct();
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'gen_off', params);
        end

        function ret = get_data(obj, varargin)
            % GET_DATA()
            % GET_DATA(...,param1,val1,param2,val2,...)
            % Get current sweep data.
            % In the BodeAnalyzer this is an alias for ``get_realtime_data`` as the data
            % is never downsampled. 
            params = struct();
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            ret = ...
            mokuctl(obj, 'get_data', params);
        end

        function ret = get_frontend(obj, channel)
            % GET_FRONTEND(channel)
            % Get the analog frontend configuration.
            % 
            % :type channel: int; {1,2}
            % :param channel: Channel for which the relay settings are being retrieved
            % 
            % :return: Array of bool with the front end configuration of channels
            %         - [0] 50 Ohm
            %         - [1] 10xAttenuation
            %         - [2] AC Coupling
            params = struct('channel',channel);
            ret = ...
            mokuctl(obj, 'get_frontend', params);
        end

        function ret = get_realtime_data(obj, varargin)
            % GET_REALTIME_DATA()
            % GET_REALTIME_DATA(...,param1,val1,param2,val2,...)
            % Get downsampled data from the instrument with low latency.
            % 
            % Returns a new :any:`InstrumentData` subclass (instrument-specific), containing
            % a version of the data that may have been downsampled from the original in order to
            % be transferred quickly.
            % 
            % This function always returns a new object at `framerate` (10Hz by default), whether
            % or not there is new data in that object. This can be verified by checking the return
            % object's *waveformid* parameter, which increments each time a new waveform is captured
            % internally.
            % 
            % The downsampled, low-latency nature of this data makes it particularly suitable for
            % plotting in real time. If you require high-accuracy, high-resolution data for analysis,
            % see `get_data`.
            % 
            % If the *wait* parameter is true (the default), this function will wait for any new
            % settings to be applied before returning. That is, if you have set a new timebase (for example),
            % calling this with *wait=True* will guarantee that the data returned has this new timebase.
            % 
            % Note that if instrument configuration is changed, a trigger event must occur before data
            % captured with that configuration set can become available. This can take an arbitrary amount
            % of time. For this reason the *timeout* should be set appropriately.
            % 
            % :type timeout: float
            % :param timeout: Maximum time to wait for new data, or *None* for indefinite.
            % 
            % :type wait: bool
            % :param wait: If *true* (default), waits for a new waveform to be captured with the most
            %         recently-applied settings, otherwise just return the most recently captured valid data.
            % 
            % :return: :any:`InstrumentData` subclass, specific to the instrument.
            params = struct();
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            ret = ...
            mokuctl(obj, 'get_realtime_data', params);
        end

        function set_defaults(obj)
            % SET_DEFAULTS()
            % Reset the Bode Analyzer to sane defaults 
            params = struct();
            mokuctl(obj, 'set_defaults', params);
        end

        function set_framerate(obj, fr)
            % SET_FRAMERATE(fr)
            % Set framerate 
            params = struct('fr',fr);
            mokuctl(obj, 'set_framerate', params);
        end

        function ret = set_frontend(obj, channel, varargin)
            % SET_FRONTEND(channel)
            % SET_FRONTEND(...,param1,val1,param2,val2,...)
            % Configures gain, coupling and termination for each channel.
            % 
            % :type channel: int; {1,2}
            % :param channel: Channel to which the settings should be applied
            % 
            % :type fiftyr: bool
            % :param fiftyr: 50Ohm termination; default is 1MOhm.
            % 
            % :type atten: bool
            % :param atten: Turn on 10x attenuation. Changes the dynamic range between 1Vpp and 10Vpp.
            % 
            % :type ac: bool
            % :param ac: AC-couple; default DC.
            params = struct('channel',channel);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            ret = ...
            mokuctl(obj, 'set_frontend', params);
        end

        function set_output(obj, ch, amplitude, varargin)
            % SET_OUTPUT(ch, amplitude)
            % SET_OUTPUT(...,param1,val1,param2,val2,...)
            % Set the output sweep amplitude.
            % 
            % .. note::
            %         Ensure that the output amplitude is set so as to not saturate the inputs.
            %         Inputs are limited to 1.0Vpp with attenuation turned off.
            % 
            % :type ch: int; {1, 2}
            % :param ch: Output channel
            % 
            % :type amplitude: float; [0.0, 2.0] Vpp
            % :param amplitude: Sweep amplitude
            % 
            % :type offset: float; [-1.0, 1.0] Volts
            % :param offset: Sweep offset
            params = struct('ch',ch,'amplitude',amplitude);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_output', params);
        end

        function set_sweep(obj, varargin)
            % SET_SWEEP()
            % SET_SWEEP(...,param1,val1,param2,val2,...)
            % Set the output sweep parameters
            % 
            % :type f_start: int; 1 <= f_start <= 120e6 Hz
            % :param f_start: Sweep start frequency
            % 
            % :type f_end: int; 1 <= f_end <= 120e6 Hz
            % :param f_end: Sweep end frequency
            % 
            % :type sweep_points: int; 32 <= sweep_points <= 512
            % :param sweep_points: Number of points in the sweep (rounded to nearest power of 2).
            % 
            % :type sweep_log: bool
            % :param sweep_log: Enable logarithmic frequency sweep scale.
            % 
            % :type averaging_time: float; sec
            % :param averaging_time: Minimum averaging time per sweep point.
            % 
            % :type settling_time: float; sec
            % :param settling_time: Minimum setting time per sweep point.
            % 
            % :type averaging_cycles: int; cycles
            % :param averaging_cycles: Minimum averaging cycles per sweep point.
            % 
            % :type settling_cycles: int; cycles
            % :param settling_cycles: Minimum settling cycles per sweep point.
            params = struct();
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_sweep', params);
        end

        function set_xmode(obj, xmode)
            % SET_XMODE(xmode)
            % Set rendering mode for the horizontal axis.
            % 
            % :type xmode: string, {'sweep','fullframe'}
            % :param xmode:
            %         Respectively; Sweep Mode (bode function sweeping across the screen)
            %         or Full Frame (like sweep, but waits for the frame to be completed).
            params = struct('xmode',xmode);
            mokuctl(obj, 'set_xmode', params);
        end

        function start_sweep(obj, varargin)
            % START_SWEEP()
            % START_SWEEP(...,param1,val1,param2,val2,...)
            % Start sweeping
            % 
            % :type single: bool
            % :param single: Enable single sweep (otherwise loop)
            params = struct();
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'start_sweep', params);
        end

        function stop_sweep(obj)
            % STOP_SWEEP()
            % Stop sweeping.
            % 
            % This will stop new data frames from being received, so ensure you implement a timeout
            % on :any:`get_data<pymoku.instruments.BodeAnalyzer.get_data>` calls. 
            params = struct();
            mokuctl(obj, 'stop_sweep', params);
        end

    end
end
