
classdef MokuFIRFilter < moku
    % MOKUFIRFILTER(IpAddr) 
    % Driver class for the Moku:FIRFilter
    %
    % MOKUFIRFILTER Properties:
    %   IP - IP address of the currently connected Moku:Lab device.
    %   Instrument - Name of the currently running instrument.
    %   Timeout - Command timeout period (sec)
    %
    % MOKUFIRFILTER Methods:
    %   commit  - Apply all modified settings.
    %   disable_output  - Disables the output of the specified FIR filter channel.
    %   get_data  - Get full-resolution data from the instrument.
    %   get_frontend  - Get the analog frontend configuration.
    %   get_realtime_data  - Get downsampled data from the instrument with low latency.
    %   get_samplerate  - :return: The current instrument sample rate (Hz) 
    %   set_control_matrix  - Configure the input control matrix specifying the input signal mixing for the specified filter channel.
    %   set_defaults  - Reset the Oscilloscope to sane defaults. 
    %   set_filter  - Set FIR filter sample rate and kernel coefficients for the specified filter channel. This will enable the specified channel output. See class documentation for 
    %   set_framerate  - Set framerate 
    %   set_frontend  - Configures gain, coupling and termination for each channel.
    %   set_gains_offsets  - Configure pre- and post-filter scales and offsets for a given filter channel.
    %   set_monitor  - Configures the specified monitor channel to view the desired filterbox signal.
    %   set_precision_mode  - Change aquisition mode between downsampling and decimation.
    %   set_samplerate  - Manually set the sample rate of the instrument.
    %   set_timebase  - Set the left- and right-hand span for the time axis.
    %   set_trigger  - Set the trigger source for the monitor channel signals. This can be either of the input or
    %   set_xmode  - Set rendering mode for the horizontal axis.
    %
    % See also moku

    methods
        function obj = MokuFIRFilter(IpAddr)
            obj@moku(IpAddr, 'firfilter');
        end

        function commit(obj)
            % COMMIT()
            % Apply all modified settings.
            % 
            % .. note::
            % 
            %     If the `autocommit` feature has been turned off, this function can be used to manually apply any instrument
            %     settings to the Moku device. These instrument settings are those configured by calling all *set_* and *gen_* type
            %     functions. Manually calling this function allows you to atomically apply many instrument settings at once.
            params = struct();
            mokuctl(obj, 'commit', params);
        end

        function disable_output(obj, ch)
            % DISABLE_OUTPUT(ch)
            % Disables the output of the specified FIR filter channel.
            % 
            % :type ch: int; {1,2}
            % :param ch: target channel
            params = struct('ch',ch);
            mokuctl(obj, 'disable_output', params);
        end

        function ret = get_data(obj, varargin)
            % GET_DATA()
            % GET_DATA(...,param1,val1,param2,val2,...)
            % Get full-resolution data from the instrument.
            % 
            % This will pause the instrument and download the entire contents of the instrument's
            % internal memory. This may include slightly more data than the instrument is set up
            % to record due to rounding of some parameters in the instrument.
            % 
            % All settings must be committed before you call this function. If *pymoku.autocommit=True*
            % (the default) then this will always be true, otherwise you will need to have called
            % :any:`commit` first.
            % 
            % The download process may take a second or so to complete. If you require high rate
            % data, e.g. for rendering a plot, see `get_realtime_data`.
            % 
            % If the *wait* parameter is true (the default), this function will wait for any new
            % settings to be applied before returning. That is, if you have set a new timebase (for example),
            % calling this with *wait=True* will guarantee that the data returned has this new timebase.
            % 
            % Note that if instrument configuration is changed, a trigger event must occur before data
            % captured with that configuration set can become available. This can take an arbitrary amount
            % of time. For this reason the *timeout* should be set appropriately.
            % 
            % :type timeout: float
            % :param timeout: Maximum time to wait for new data, or *None* for indefinite.
            % 
            % :type wait: bool
            % :param wait: If *true* (default), waits for a new waveform to be captured with the most
            %         recently-applied settings, otherwise just return the most recently captured valid data.
            % 
            % :return: :any:`InstrumentData` subclass, specific to the instrument.
            params = struct();
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            ret = ...
            mokuctl(obj, 'get_data', params);
        end

        function ret = get_frontend(obj, channel)
            % GET_FRONTEND(channel)
            % Get the analog frontend configuration.
            % 
            % :type channel: int; {1,2}
            % :param channel: Channel for which the relay settings are being retrieved
            % 
            % :return: Array of bool with the front end configuration of channels
            %         - [0] 50 Ohm
            %         - [1] 10xAttenuation
            %         - [2] AC Coupling
            params = struct('channel',channel);
            ret = ...
            mokuctl(obj, 'get_frontend', params);
        end

        function ret = get_realtime_data(obj, varargin)
            % GET_REALTIME_DATA()
            % GET_REALTIME_DATA(...,param1,val1,param2,val2,...)
            % Get downsampled data from the instrument with low latency.
            % 
            % Returns a new :any:`InstrumentData` subclass (instrument-specific), containing
            % a version of the data that may have been downsampled from the original in order to
            % be transferred quickly.
            % 
            % This function always returns a new object at `framerate` (10Hz by default), whether
            % or not there is new data in that object. This can be verified by checking the return
            % object's *waveformid* parameter, which increments each time a new waveform is captured
            % internally.
            % 
            % The downsampled, low-latency nature of this data makes it particularly suitable for
            % plotting in real time. If you require high-accuracy, high-resolution data for analysis,
            % see `get_data`.
            % 
            % If the *wait* parameter is true (the default), this function will wait for any new
            % settings to be applied before returning. That is, if you have set a new timebase (for example),
            % calling this with *wait=True* will guarantee that the data returned has this new timebase.
            % 
            % Note that if instrument configuration is changed, a trigger event must occur before data
            % captured with that configuration set can become available. This can take an arbitrary amount
            % of time. For this reason the *timeout* should be set appropriately.
            % 
            % :type timeout: float
            % :param timeout: Maximum time to wait for new data, or *None* for indefinite.
            % 
            % :type wait: bool
            % :param wait: If *true* (default), waits for a new waveform to be captured with the most
            %         recently-applied settings, otherwise just return the most recently captured valid data.
            % 
            % :return: :any:`InstrumentData` subclass, specific to the instrument.
            params = struct();
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            ret = ...
            mokuctl(obj, 'get_realtime_data', params);
        end

        function ret = get_samplerate(obj)
            % GET_SAMPLERATE()
            % :return: The current instrument sample rate (Hz) 
            params = struct();
            ret = ...
            mokuctl(obj, 'get_samplerate', params);
        end

        function set_control_matrix(obj, ch, scale_in1, scale_in2)
            % SET_CONTROL_MATRIX(ch, scale_in1, scale_in2)
            % Configure the input control matrix specifying the input signal mixing for the specified filter channel.
            % 
            % Input mixing allows a filter channel to act on a linear combination of the two input signals.
            % 
            % :type ch: int, {1, 2}
            % :param ch: target filter channel
            % 
            % :type scale_in1: float, [-20, 20]
            % :param scale_in1: linear scale factor of input 1 signal added to target filter channel input.
            %         To avoid quantization, use at most one decimal place.
            % 
            % :type scale_in2: float, [-20, 20] 
            % :param scale_in2: linear scale factor of input 2 signal added to target filter channel input.
            %         To avoid quantization, use at most one decimal place.
            params = struct('ch',ch,'scale_in1',scale_in1,'scale_in2',scale_in2);
            mokuctl(obj, 'set_control_matrix', params);
        end

        function set_defaults(obj)
            % SET_DEFAULTS()
            % Reset the Oscilloscope to sane defaults. 
            params = struct();
            mokuctl(obj, 'set_defaults', params);
        end

        function set_filter(obj, ch, decimation_factor, filter_coefficients)
            % SET_FILTER(ch, decimation_factor, filter_coefficients)
            % Set FIR filter sample rate and kernel coefficients for the specified filter channel. This will enable the specified channel output. See class documentation for 
            % information on the `filter_coefficients` array formatting and how `decimation_factor` relates to instrument sample rate. 
            % 
            % :type ch: int; {1,2}
            % :param ch: Filter channel.
            % 
            % :type decimation_factor: int; [3,10]
            % :param decimation_factor: the binary exponent *n* specifying the filter sample rate: :math:`Fs = 125 MHz / 2^n`.
            % 
            % :type filter_coefficients: array(float)
            % :param filter_coefficients: array of FIR filter coefficients. The length of the array must not exceed :math:`N = min(29*2^n, 14819)`.
            params = struct('ch',ch,'decimation_factor',decimation_factor,'filter_coefficients',filter_coefficients);
            mokuctl(obj, 'set_filter', params);
        end

        function set_framerate(obj, fr)
            % SET_FRAMERATE(fr)
            % Set framerate 
            params = struct('fr',fr);
            mokuctl(obj, 'set_framerate', params);
        end

        function ret = set_frontend(obj, channel, varargin)
            % SET_FRONTEND(channel)
            % SET_FRONTEND(...,param1,val1,param2,val2,...)
            % Configures gain, coupling and termination for each channel.
            % 
            % :type channel: int; {1,2}
            % :param channel: Channel to which the settings should be applied
            % 
            % :type fiftyr: bool
            % :param fiftyr: 50Ohm termination; default is 1MOhm.
            % 
            % :type atten: bool
            % :param atten: Turn on 10x attenuation. Changes the dynamic range between 1Vpp and 10Vpp.
            % 
            % :type ac: bool
            % :param ac: AC-couple; default DC.
            params = struct('channel',channel);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            ret = ...
            mokuctl(obj, 'set_frontend', params);
        end

        function set_gains_offsets(obj, ch, varargin)
            % SET_GAINS_OFFSETS(ch)
            % SET_GAINS_OFFSETS(...,param1,val1,param2,val2,...)
            % Configure pre- and post-filter scales and offsets for a given filter channel.
            % 
            % .. note::
            %         The overall output gain of the instrument is the product of the gain of the filter, set by the filter coefficients,
            %         and the input/output stage gain set here.
            % 
            % :type ch: int, {1,2}
            % :param ch: target filter channel
            % 
            % :type input_gain, output_gain: float, linear scalar, [-100,100]
            % :param input_gain, output_gain: channel scalars before and after the FIR filter
            % 
            % :type input_offset: float, volts, [-1.0,1.0]
            % :param input_offset: channel offset before the FIR filter
            % 
            % :type output_offset: float, volts, [-2.0,2.0]
            % :param output_offset: channel offset after the FIR filter
            params = struct('ch',ch);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_gains_offsets', params);
        end

        function set_monitor(obj, monitor_ch, source, varargin)
            % SET_MONITOR(monitor_ch, source)
            % SET_MONITOR(...,param1,val1,param2,val2,...)
            % Configures the specified monitor channel to view the desired filterbox signal.
            % 
            % There are two 12-bit monitoring channels available, 'a' and 'b'; each of these can 
            % be assigned to source signals from any of the internal filterbox monitoring points. 
            % Signals larger than 12-bits must be either truncated or clipped to the allowed size.
            % 
            % The source is one of:
            %         - **adc1**      : Channel 1 ADC input
            %         - **in1**       : Filter Channel 1 input (after mixing, offset and scaling)
            %         - **out1**      : Filter Channel 1 output
            %         - **adc2**      : Channel 2 ADC Input
            %         - **in2**       : Filter Channel 2 input (after mixing, offset and scaling)
            %         - **out2**      : Filter Channel 2 output
            % 
            % :type monitor_ch: str; {'a','b'}
            % :param monitor_ch: Monitor channel
            % 
            % :type source: str; {'adc1', 'in1', 'out1', 'adc2', 'in2', 'out2'}
            % :param source: Signal to connect to the monitor channel
            % 
            % :type clip: bool;
            % :param clip: Enable signal clipping to the allowed size (truncate is default).
            params = struct('monitor_ch',monitor_ch,'source',source);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_monitor', params);
        end

        function set_precision_mode(obj, state)
            % SET_PRECISION_MODE(state)
            % Change aquisition mode between downsampling and decimation.
            % Precision mode, a.k.a Decimation, samples at full rate and applies a low-pass filter to the data. This improves
            % precision. Normal mode works by direct downsampling, throwing away points it doesn't need.
            % 
            % :param state: Select Precision Mode
            % :type state: bool
            params = struct('state',state);
            mokuctl(obj, 'set_precision_mode', params);
        end

        function set_samplerate(obj, samplerate, varargin)
            % SET_SAMPLERATE(samplerate)
            % SET_SAMPLERATE(...,param1,val1,param2,val2,...)
            % Manually set the sample rate of the instrument.
            % 
            % The sample rate is automatically calculated and set in :any:`set_timebase`.
            % 
            % This interface allows you to specify the rate at which data is sampled, and set
            % a trigger offset in number of samples. This interface is useful for datalogging and capturing
            % of data frames.
            % 
            % :type samplerate: float; *0 < samplerate <= MAX_SAMPLERATE smp/s*
            % :param samplerate: Target samples per second. Will get rounded to the nearest allowable unit.
            % 
            % :type trigger_offset: int; *-2^16 < trigger_offset < 2^31*
            % :param trigger_offset: Number of samples before (-) or after (+) the trigger point to start capturing.
            % 
            % :raises ValueOutOfRangeException: if either parameter is out of range.
            params = struct('samplerate',samplerate);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_samplerate', params);
        end

        function set_timebase(obj, t1, t2)
            % SET_TIMEBASE(t1, t2)
            % Set the left- and right-hand span for the time axis.
            % Units are seconds relative to the trigger point.
            % 
            % :type t1: float
            % :param t1:
            %         Time, in seconds, from the trigger point to the left of screen. This may be negative (trigger on-screen)
            %         or positive (trigger off the left of screen).
            % 
            % :type t2: float
            % :param t2: As *t1* but to the right of screen.
            % 
            % :raises InvalidConfigurationException: if the timebase is backwards or zero.
            params = struct('t1',t1,'t2',t2);
            mokuctl(obj, 'set_timebase', params);
        end

        function set_trigger(obj, source, edge, level, varargin)
            % SET_TRIGGER(source, edge, level)
            % SET_TRIGGER(...,param1,val1,param2,val2,...)
            % Set the trigger source for the monitor channel signals. This can be either of the input or
            % monitor signals, or the external input.
            % 
            % :type source: string, {'in1','in2','A','B','ext'}
            % :param source: Trigger Source. May be either an input or monitor channel (as set by 
            %                 :py:meth:`~pymoku.instruments.FIRFilter.set_monitor`), or external. External refers 
            %                 to the back-panel connector of the same name, allowing triggering from an 
            %                 externally-generated digital [LV]TTL or CMOS signal.
            % 
            % :type edge: string, {'rising','falling','both'}
            % :param edge: Which edge to trigger on. In Pulse Width modes this specifies whether the pulse is positive (rising)
            %                 or negative (falling), with the 'both' option being invalid.
            % 
            % :type level: float, [-10.0, 10.0] volts
            % :param level: Trigger level
            % 
            % :type minwidth: float, seconds
            % :param minwidth: Minimum Pulse Width. 0 <= minwidth < (2^32/samplerate). Can't be used with maxwidth.
            % 
            % :type maxwidth: float, seconds
            % :param maxwidth: Maximum Pulse Width. 0 <= maxwidth < (2^32/samplerate). Can't be used with minwidth.
            % 
            % :type hysteresis: float, [100e-6, 1.0] volts
            % :param hysteresis: Hysteresis around trigger point.
            % 
            % :type hf_reject: bool
            % :param hf_reject: Enable high-frequency noise rejection
            % 
            % :type mode: string, {'auto', 'normal'}
            % :param mode: Trigger mode.
            params = struct('source',source,'edge',edge,'level',level);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_trigger', params);
        end

        function set_xmode(obj, xmode)
            % SET_XMODE(xmode)
            % Set rendering mode for the horizontal axis.
            % 
            % :type xmode: string, {'roll','sweep','fullframe'}
            % :param xmode:
            %         Respectively; Roll Mode (scrolling), Sweep Mode (normal oscilloscope trace sweeping across the screen)
            %         or Full Frame (like sweep, but waits for the frame to be completed).
            params = struct('xmode',xmode);
            mokuctl(obj, 'set_xmode', params);
        end

    end
end
