
classdef MokuLockInAmp < moku
    % MOKULOCKINAMP(IpAddr) 
    % Driver class for the Moku:LockInAmp
    %
    % MOKULOCKINAMP Properties:
    %   IP - IP address of the currently connected Moku:Lab device.
    %   Instrument - Name of the currently running instrument.
    %   Timeout - Command timeout period (sec)
    %
    % MOKULOCKINAMP Methods:
    %   commit  - Apply all modified settings.
    %   enable_input  - Enable or disable the PID channel input(s).
    %   enable_output  - Enable or disable the PID channel output(s).
    %   get_data  - Get full-resolution data from the instrument.
    %   get_frontend  - Get the analog frontend configuration.
    %   get_realtime_data  - Get downsampled data from the instrument with low latency.
    %   get_samplerate  - :return: The current instrument sample rate (Hz) 
    %   set_by_frequency  - Set which lock-in channel the PID is on and configure it using frequency domain parameters.
    %   set_by_gain  - Set which lock-in channel the PID is on and configure it using gain parameters.
    %   set_control_matrix  - DEPRECATED 
    %   set_defaults  - Reset the lockinamp to sane defaults. 
    %   set_demodulation  - Configure the demodulation stage.
    %   set_filter  - Set the low-pass filter parameters.
    %   set_framerate  - Set framerate 
    %   set_frontend  - Configures gain, coupling and termination for each channel.
    %   set_gain  - Sets the gain stage to be on the specified lock-in channel, and configures its gain.
    %   set_input_gain  - Set the main input gain (Input Channel 1).
    %   set_lo_output  - Configure local oscillator output.
    %   set_monitor  - Select the point inside the lockin amplifier to monitor.
    %   set_outputs  - Configures the main (Channel 1) and auxillary (Channel 2) output signals of the Lock-In.
    %   set_pid_by_frequency  - Set which lock-in channel the PID is on and configure it using frequency domain parameters.
    %   set_pid_by_gain  - Set which lock-in channel the PID is on and configure it using gain parameters.
    %   set_precision_mode  - Change aquisition mode between downsampling and decimation.
    %   set_samplerate  - Manually set the sample rate of the instrument.
    %   set_timebase  - Set the left- and right-hand span for the time axis.
    %   set_trigger  - Set the trigger source for the monitor channel signals. This can be either of the input or
    %   set_xmode  - Set rendering mode for the horizontal axis.
    %
    % See also moku

    methods
        function obj = MokuLockInAmp(IpAddr)
            obj@moku(IpAddr, 'lockinamp');
        end

        function commit(obj)
            % COMMIT()
            % Apply all modified settings.
            % 
            % .. note::
            % 
            %     If the `autocommit` feature has been turned off, this function can be used to manually apply any instrument
            %     settings to the Moku device. These instrument settings are those configured by calling all *set_* and *gen_* type
            %     functions. Manually calling this function allows you to atomically apply many instrument settings at once.
            params = struct();
            mokuctl(obj, 'commit', params);
        end

        function enable_input(obj, varargin)
            % ENABLE_INPUT()
            % ENABLE_INPUT(...,param1,val1,param2,val2,...)
            % Enable or disable the PID channel input(s).
            % 
            % If *ch* is None (the default), both channels will be acted upon,
            % otherwise just the one specified by the argument.
            % 
            % :type ch: int; {1,2} or None
            % :param ch: Output channel, or both.
            % 
            % :type en: bool
            % :param en: Enable the specified input channel(s).
            % 
            % :raises InvalidParameterException: Invalid parameters
            params = struct();
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'enable_input', params);
        end

        function enable_output(obj, varargin)
            % ENABLE_OUTPUT()
            % ENABLE_OUTPUT(...,param1,val1,param2,val2,...)
            % Enable or disable the PID channel output(s).
            % 
            % If *ch* is None (the default), both channels will be acted upon,
            % otherwise just the one specified by the argument.
            % 
            % :type ch: int; {1,2} or None
            % :param ch: Output channel, or both.
            % 
            % :type en: bool
            % :param en: Enable the specified output channel(s).
            % 
            % :raises InvalidParameterException: Invalid parameters
            params = struct();
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'enable_output', params);
        end

        function ret = get_data(obj, varargin)
            % GET_DATA()
            % GET_DATA(...,param1,val1,param2,val2,...)
            % Get full-resolution data from the instrument.
            % 
            % This will pause the instrument and download the entire contents of the instrument's
            % internal memory. This may include slightly more data than the instrument is set up
            % to record due to rounding of some parameters in the instrument.
            % 
            % All settings must be committed before you call this function. If *pymoku.autocommit=True*
            % (the default) then this will always be true, otherwise you will need to have called
            % :any:`commit` first.
            % 
            % The download process may take a second or so to complete. If you require high rate
            % data, e.g. for rendering a plot, see `get_realtime_data`.
            % 
            % If the *wait* parameter is true (the default), this function will wait for any new
            % settings to be applied before returning. That is, if you have set a new timebase (for example),
            % calling this with *wait=True* will guarantee that the data returned has this new timebase.
            % 
            % Note that if instrument configuration is changed, a trigger event must occur before data
            % captured with that configuration set can become available. This can take an arbitrary amount
            % of time. For this reason the *timeout* should be set appropriately.
            % 
            % :type timeout: float
            % :param timeout: Maximum time to wait for new data, or *None* for indefinite.
            % 
            % :type wait: bool
            % :param wait: If *true* (default), waits for a new waveform to be captured with the most
            %         recently-applied settings, otherwise just return the most recently captured valid data.
            % 
            % :return: :any:`InstrumentData` subclass, specific to the instrument.
            params = struct();
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            ret = ...
            mokuctl(obj, 'get_data', params);
        end

        function ret = get_frontend(obj, channel)
            % GET_FRONTEND(channel)
            % Get the analog frontend configuration.
            % 
            % :type channel: int; {1,2}
            % :param channel: Channel for which the relay settings are being retrieved
            % 
            % :return: Array of bool with the front end configuration of channels
            %         - [0] 50 Ohm
            %         - [1] 10xAttenuation
            %         - [2] AC Coupling
            params = struct('channel',channel);
            ret = ...
            mokuctl(obj, 'get_frontend', params);
        end

        function ret = get_realtime_data(obj, varargin)
            % GET_REALTIME_DATA()
            % GET_REALTIME_DATA(...,param1,val1,param2,val2,...)
            % Get downsampled data from the instrument with low latency.
            % 
            % Returns a new :any:`InstrumentData` subclass (instrument-specific), containing
            % a version of the data that may have been downsampled from the original in order to
            % be transferred quickly.
            % 
            % This function always returns a new object at `framerate` (10Hz by default), whether
            % or not there is new data in that object. This can be verified by checking the return
            % object's *waveformid* parameter, which increments each time a new waveform is captured
            % internally.
            % 
            % The downsampled, low-latency nature of this data makes it particularly suitable for
            % plotting in real time. If you require high-accuracy, high-resolution data for analysis,
            % see `get_data`.
            % 
            % If the *wait* parameter is true (the default), this function will wait for any new
            % settings to be applied before returning. That is, if you have set a new timebase (for example),
            % calling this with *wait=True* will guarantee that the data returned has this new timebase.
            % 
            % Note that if instrument configuration is changed, a trigger event must occur before data
            % captured with that configuration set can become available. This can take an arbitrary amount
            % of time. For this reason the *timeout* should be set appropriately.
            % 
            % :type timeout: float
            % :param timeout: Maximum time to wait for new data, or *None* for indefinite.
            % 
            % :type wait: bool
            % :param wait: If *true* (default), waits for a new waveform to be captured with the most
            %         recently-applied settings, otherwise just return the most recently captured valid data.
            % 
            % :return: :any:`InstrumentData` subclass, specific to the instrument.
            params = struct();
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            ret = ...
            mokuctl(obj, 'get_realtime_data', params);
        end

        function ret = get_samplerate(obj)
            % GET_SAMPLERATE()
            % :return: The current instrument sample rate (Hz) 
            params = struct();
            ret = ...
            mokuctl(obj, 'get_samplerate', params);
        end

        function set_by_frequency(obj, lia_ch, varargin)
            % SET_BY_FREQUENCY(lia_ch)
            % SET_BY_FREQUENCY(...,param1,val1,param2,val2,...)
            % Set which lock-in channel the PID is on and configure it using frequency domain parameters.
            % 
            % This sets the gain stage to be on the opposite channel.
            % 
            % :type ch: string; {'main','aux'}
            % :param ch: Lock-in channel name to put PID on. 
            % 
            % :type kp: float; [-1e3,1e3]
            % :param kp: Proportional gain factor
            % 
            % :type i_xover: float; [1e-3,1e6] Hz
            % :param i_xover: Integrator crossover frequency
            % 
            % :type d_xover: float; [1,10e6] Hz
            % :param d_xover: Differentiator crossover frequency
            % 
            % :type ii_xover: float; [1, 1e6] Hz
            % :param ii_xover: Second integrator crossover frequency
            % 
            % :type si: float; float; [-1e3,1e3]
            % :param si: Integrator gain saturation
            % 
            % :type sd: float; [-1e3,1e3]
            % :param sd: Differentiator gain saturation
            % 
            % :type in_offset: float; [-1.0,1.0] V
            % :param in_offset: Input signal offset
            % 
            % :type out_offset: float; [-1.0, 1.0] V
            % :type out_offset: Output signal offset
            % 
            % :raises InvalidConfigurationException: if the configuration of PID gains is not possible.
            params = struct('lia_ch',lia_ch);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_by_frequency', params);
        end

        function set_by_gain(obj, lia_ch, g, varargin)
            % SET_BY_GAIN(lia_ch, g)
            % SET_BY_GAIN(...,param1,val1,param2,val2,...)
            % Set which lock-in channel the PID is on and configure it using gain parameters.
            % 
            % This sets the gain stage to be on the opposite channel.
            % 
            % :type ch: string; {'main','aux'}
            % :param ch: Lock-in channel name to put PID on
            % 
            % :type g: float; [0,2^16 - 1]
            % :param g: Gain
            % 
            % :type kp: float; [-1e3,1e3]
            % :param kp: Proportional gain factor
            % 
            % :type ki: float;
            % :param ki: Integrator gain factor
            % 
            % :type kd: float;
            % :param kd: Differentiator gain factor
            % 
            % :type kii: float;
            % :param kii: Second integrator gain factor
            % 
            % :type si: float; float; [-1e3,1e3]
            % :param si: Integrator gain saturation
            % 
            % :type sd: float; [-1e3,1e3]
            % :param sd: Differentiator gain saturation
            % 
            % :type in_offset: float; [-1.0,1.0] V
            % :param in_offset: Input signal offset
            % 
            % :type out_offset: float; [-1.0, 1.0] V
            % :type out_offset: Output signal offset
            % 
            % :raises InvalidConfigurationException: if the configuration of PID gains is not possible.
            params = struct('lia_ch',lia_ch,'g',g);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_by_gain', params);
        end

        function set_control_matrix(obj)
            % SET_CONTROL_MATRIX()
            % DEPRECATED 
            params = struct();
            mokuctl(obj, 'set_control_matrix', params);
        end

        function set_defaults(obj)
            % SET_DEFAULTS()
            % Reset the lockinamp to sane defaults. 
            params = struct();
            mokuctl(obj, 'set_defaults', params);
        end

        function set_demodulation(obj, mode, varargin)
            % SET_DEMODULATION(mode)
            % SET_DEMODULATION(...,param1,val1,param2,val2,...)
            % Configure the demodulation stage.
            % 
            % The mode is one of:
            %         - **internal** : for an internally set local oscillator
            %         - **external** : to directly use an external signal for demodulation (Note: Q is not selectable in this mode)
            %         - **external_pll** : to use an external signal for demodulation after running it through an internal PLL.
            % 
            % .. note::
            %   When 'external' is used (that is, without a PLL), the Lock-in Amplifier doesn't know the frequency and therefore
            %   can't form the quadrature for full I/Q demodulation. This in turn means it can't distinguish I from Q, X from Y,
            %   or form R/Theta. This limits the choices for signals that can be output on the Main and AUX channels to ones not
            %   formed from the quadrature signal.
            % 
            %   An exception will be raised if you attempt to set the demodulation to 'external' while viewing one of these signals.
            % 
            % :type mode: string; {'internal', 'external', 'external_pll'}
            % :param mode: Demodulation mode
            % 
            % :type frequency: float; [0, 200e6] Hz
            % :param frequency: Internal demodulation signal frequency (ignored for all 'external' modes)
            % 
            % :type phase: float; [0, 360] deg
            % :param phase: Internal demodulation signal phase (ignored in 'external' mode)
            % 
            % :type output_amplitude: float; [0.0, 2.0] Vpp
            % :param output_amplitude: Output amplitude of the demodulation signal when auxillary channel set to output `demod`.
            params = struct('mode',mode);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_demodulation', params);
        end

        function set_filter(obj, f_corner, order)
            % SET_FILTER(f_corner, order)
            % Set the low-pass filter parameters.
            % 
            % :type f_corner: float
            % :param f_corner: Corner frequency of the low-pass filter (Hz)
            % 
            % :type order: int; [0, 1, 2]
            % :param order: filter order; 0 (bypass), first- or second-order.
            params = struct('f_corner',f_corner,'order',order);
            mokuctl(obj, 'set_filter', params);
        end

        function set_framerate(obj, fr)
            % SET_FRAMERATE(fr)
            % Set framerate 
            params = struct('fr',fr);
            mokuctl(obj, 'set_framerate', params);
        end

        function ret = set_frontend(obj, channel, varargin)
            % SET_FRONTEND(channel)
            % SET_FRONTEND(...,param1,val1,param2,val2,...)
            % Configures gain, coupling and termination for each channel.
            % 
            % :type channel: int; {1,2}
            % :param channel: Channel to which the settings should be applied
            % 
            % :type fiftyr: bool
            % :param fiftyr: 50Ohm termination; default is 1MOhm.
            % 
            % :type atten: bool
            % :param atten: Turn on 10x attenuation. Changes the dynamic range between 1Vpp and 10Vpp.
            % 
            % :type ac: bool
            % :param ac: AC-couple; default DC.
            params = struct('channel',channel);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            ret = ...
            mokuctl(obj, 'set_frontend', params);
        end

        function set_gain(obj, lia_ch, g)
            % SET_GAIN(lia_ch, g)
            % Sets the gain stage to be on the specified lock-in channel, and configures its gain.
            % 
            % This sets the PID stage to be on the opposite channel.
            % 
            % :type lia_ch: string; {'main','aux'}
            % :param lia_ch: Channel name
            % 
            % :type g: float; [0, 2^16 - 1]
            % :param g: Gain
            params = struct('lia_ch',lia_ch,'g',g);
            mokuctl(obj, 'set_gain', params);
        end

        function set_input_gain(obj, varargin)
            % SET_INPUT_GAIN()
            % SET_INPUT_GAIN(...,param1,val1,param2,val2,...)
            % Set the main input gain (Input Channel 1).
            % 
            % :type gain: int; {-20, 0, 24, 48} dB
            % :param gain: Input gain
            params = struct();
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_input_gain', params);
        end

        function set_lo_output(obj, amplitude, frequency, phase)
            % SET_LO_OUTPUT(amplitude, frequency, phase)
            % Configure local oscillator output.
            % 
            % This output is available on Channel 2 of the Moku:Lab.
            % 
            % :type amplitude: float; [0.0, 2.0] Vpp
            % :param amplitude: Amplitude
            % 
            % :type frequency: float; [0, 200e6] Hz
            % :param frequency: Frequency
            % 
            % :type phase: float; [0, 360] deg
            % :param phase: Phase
            params = struct('amplitude',amplitude,'frequency',frequency,'phase',phase);
            mokuctl(obj, 'set_lo_output', params);
        end

        function set_monitor(obj, monitor_ch, source)
            % SET_MONITOR(monitor_ch, source)
            % Select the point inside the lockin amplifier to monitor.
            % 
            % There are two monitoring channels available, 'A' and 'B'; you can mux any of the internal
            % monitoring points to either of these channels.
            % 
            % The source is one of:
            %         - **none**: Disable monitor channel
            %         - **in1**, **in2**: Input Channel 1/2
            %         - **main**: Lock-in output (Output Channel 1)
            %         - **aux**: Auxillary output (Output Channel 2)
            %         - **demod**: Demodulation signal input to mixer
            %         - **i**, **q**: Mixer I and Q channels respectively.
            % 
            % :type monitor_ch: string; {'A','B'}
            % :param monitor_ch: Monitor channel
            % :type source: string; {'none','in1','in2','main','aux','demod','i','q'}
            % :param source: Signal to monitor
            params = struct('monitor_ch',monitor_ch,'source',source);
            mokuctl(obj, 'set_monitor', params);
        end

        function set_outputs(obj, main, aux, varargin)
            % SET_OUTPUTS(main, aux)
            % SET_OUTPUTS(...,param1,val1,param2,val2,...)
            % Configures the main (Channel 1) and auxillary (Channel 2) output signals of the Lock-In.
            % 
            % .. note::
            %   When 'external' demodulation is used (that is, without a PLL), the Lock-in Amplifier doesn't know the frequency and therefore
            %   can't form the quadrature for full I/Q demodulation. This in turn means it can't distinguish I from Q, X from Y,
            %   or form R/Theta. This limits the choices for signals that can be output on the AUX channel to ones not from the
            %   Lock-in logic (e.g. demodulation source, auxilliary sine wave etc).
            % 
            %   An exception will be raised if you attempt to set the auxilliary channel to view aa signal from the Lock-in logic while
            %   external demodulation is enabled.
            % 
            % :type main: string; {'x', 'y', 'r', 'theta', 'offset', 'none'}
            % :param main: Main output signal
            % 
            % :type aux: string; {'x', 'y', 'r', theta', 'sine', 'demod', 'offset', 'none'}
            % :param aux: Auxillary output signal
            % 
            % :type main_offset: float; [-1.0, 1.0] V
            % :param main_offset: Main output offset
            % 
            % :type aux_offset: float; [-1.0, 1.0] V
            % :param aux_offset: Auxillary output offset
            params = struct('main',main,'aux',aux);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_outputs', params);
        end

        function set_pid_by_frequency(obj, lia_ch, varargin)
            % SET_PID_BY_FREQUENCY(lia_ch)
            % SET_PID_BY_FREQUENCY(...,param1,val1,param2,val2,...)
            % Set which lock-in channel the PID is on and configure it using frequency domain parameters.
            % 
            % This sets the gain stage to be on the opposite channel.
            % 
            % :type ch: string; {'main','aux'}
            % :param ch: Lock-in channel name to put PID on. 
            % 
            % :type kp: float; [-1e3,1e3]
            % :param kp: Proportional gain factor
            % 
            % :type i_xover: float; [1e-3,1e6] Hz
            % :param i_xover: Integrator crossover frequency
            % 
            % :type d_xover: float; [1,10e6] Hz
            % :param d_xover: Differentiator crossover frequency
            % 
            % :type ii_xover: float; [1, 1e6] Hz
            % :param ii_xover: Second integrator crossover frequency
            % 
            % :type si: float; float; [-1e3,1e3]
            % :param si: Integrator gain saturation
            % 
            % :type sd: float; [-1e3,1e3]
            % :param sd: Differentiator gain saturation
            % 
            % :type in_offset: float; [-1.0,1.0] V
            % :param in_offset: Input signal offset
            % 
            % :type out_offset: float; [-1.0, 1.0] V
            % :type out_offset: Output signal offset
            % 
            % :raises InvalidConfigurationException: if the configuration of PID gains is not possible.
            params = struct('lia_ch',lia_ch);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_pid_by_frequency', params);
        end

        function set_pid_by_gain(obj, lia_ch, g, varargin)
            % SET_PID_BY_GAIN(lia_ch, g)
            % SET_PID_BY_GAIN(...,param1,val1,param2,val2,...)
            % Set which lock-in channel the PID is on and configure it using gain parameters.
            % 
            % This sets the gain stage to be on the opposite channel.
            % 
            % :type ch: string; {'main','aux'}
            % :param ch: Lock-in channel name to put PID on
            % 
            % :type g: float; [0,2^16 - 1]
            % :param g: Gain
            % 
            % :type kp: float; [-1e3,1e3]
            % :param kp: Proportional gain factor
            % 
            % :type ki: float;
            % :param ki: Integrator gain factor
            % 
            % :type kd: float;
            % :param kd: Differentiator gain factor
            % 
            % :type kii: float;
            % :param kii: Second integrator gain factor
            % 
            % :type si: float; float; [-1e3,1e3]
            % :param si: Integrator gain saturation
            % 
            % :type sd: float; [-1e3,1e3]
            % :param sd: Differentiator gain saturation
            % 
            % :type in_offset: float; [-1.0,1.0] V
            % :param in_offset: Input signal offset
            % 
            % :type out_offset: float; [-1.0, 1.0] V
            % :type out_offset: Output signal offset
            % 
            % :raises InvalidConfigurationException: if the configuration of PID gains is not possible.
            params = struct('lia_ch',lia_ch,'g',g);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_pid_by_gain', params);
        end

        function set_precision_mode(obj, state)
            % SET_PRECISION_MODE(state)
            % Change aquisition mode between downsampling and decimation.
            % Precision mode, a.k.a Decimation, samples at full rate and applies a low-pass filter to the data. This improves
            % precision. Normal mode works by direct downsampling, throwing away points it doesn't need.
            % 
            % :param state: Select Precision Mode
            % :type state: bool
            params = struct('state',state);
            mokuctl(obj, 'set_precision_mode', params);
        end

        function set_samplerate(obj, samplerate, varargin)
            % SET_SAMPLERATE(samplerate)
            % SET_SAMPLERATE(...,param1,val1,param2,val2,...)
            % Manually set the sample rate of the instrument.
            % 
            % The sample rate is automatically calculated and set in :any:`set_timebase`.
            % 
            % This interface allows you to specify the rate at which data is sampled, and set
            % a trigger offset in number of samples. This interface is useful for datalogging and capturing
            % of data frames.
            % 
            % :type samplerate: float; *0 < samplerate <= MAX_SAMPLERATE smp/s*
            % :param samplerate: Target samples per second. Will get rounded to the nearest allowable unit.
            % 
            % :type trigger_offset: int; *-2^16 < trigger_offset < 2^31*
            % :param trigger_offset: Number of samples before (-) or after (+) the trigger point to start capturing.
            % 
            % :raises ValueOutOfRangeException: if either parameter is out of range.
            params = struct('samplerate',samplerate);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_samplerate', params);
        end

        function set_timebase(obj, t1, t2)
            % SET_TIMEBASE(t1, t2)
            % Set the left- and right-hand span for the time axis.
            % Units are seconds relative to the trigger point.
            % 
            % :type t1: float
            % :param t1:
            %         Time, in seconds, from the trigger point to the left of screen. This may be negative (trigger on-screen)
            %         or positive (trigger off the left of screen).
            % 
            % :type t2: float
            % :param t2: As *t1* but to the right of screen.
            % 
            % :raises InvalidConfigurationException: if the timebase is backwards or zero.
            params = struct('t1',t1,'t2',t2);
            mokuctl(obj, 'set_timebase', params);
        end

        function set_trigger(obj, source, edge, level, varargin)
            % SET_TRIGGER(source, edge, level)
            % SET_TRIGGER(...,param1,val1,param2,val2,...)
            % Set the trigger source for the monitor channel signals. This can be either of the input or
            % monitor signals, or the external input.
            % 
            % :type source: string, {'in1','in2','A','B','ext'}
            % :param source: Trigger Source. May be either an input or monitor channel (as set by 
            %                 :py:meth:`~pymoku.instruments.LockInAmp.set_monitor`), or external. External refers 
            %                 to the back-panel connector of the same name, allowing triggering from an 
            %                 externally-generated digital [LV]TTL or CMOS signal.
            % 
            % :type edge: string, {'rising','falling','both'}
            % :param edge: Which edge to trigger on. In Pulse Width modes this specifies whether the pulse is positive (rising)
            %                 or negative (falling), with the 'both' option being invalid.
            % 
            % :type level: float, [-10.0, 10.0] volts
            % :param level: Trigger level
            % 
            % :type minwidth: float, seconds
            % :param minwidth: Minimum Pulse Width. 0 <= minwidth < (2^32/samplerate). Can't be used with maxwidth.
            % 
            % :type maxwidth: float, seconds
            % :param maxwidth: Maximum Pulse Width. 0 <= maxwidth < (2^32/samplerate). Can't be used with minwidth.
            % 
            % :type hysteresis: float, [100e-6, 1.0] volts
            % :param hysteresis: Hysteresis around trigger point.
            % 
            % :type hf_reject: bool
            % :param hf_reject: Enable high-frequency noise rejection
            % 
            % :type mode: string, {'auto', 'normal'}
            % :param mode: Trigger mode.
            params = struct('source',source,'edge',edge,'level',level);
            kwargs = varargin;
            params = moku.params_to_struct(params, kwargs);
            mokuctl(obj, 'set_trigger', params);
        end

        function set_xmode(obj, xmode)
            % SET_XMODE(xmode)
            % Set rendering mode for the horizontal axis.
            % 
            % :type xmode: string, {'roll','sweep','fullframe'}
            % :param xmode:
            %         Respectively; Roll Mode (scrolling), Sweep Mode (normal oscilloscope trace sweeping across the screen)
            %         or Full Frame (like sweep, but waits for the frame to be completed).
            params = struct('xmode',xmode);
            mokuctl(obj, 'set_xmode', params);
        end

    end
end
